@@ -37,76 +37,21 @@ char c_msg_id[] = "c_msg.c v2.0 (c) 1988 University of Oulu, Computing Center\
 #endif
 
 #include "common.h"
+#include "sys.h"
 #include "msg.h"
-#if 0
-#ifndef SOL20
+#include "irc.h"
 #include "h.h"
-#endif /* SOL20	Bad m_names and m_private prototypes	Vesa */
-#endif /* 0	Nobody needs it. */
 
 char	mybuf[513];
 
-extern	anIgnore *find_ignore();
 extern	char	*last_to_me(), userhost[];
 extern	aClient	*client, me; /* 'me' from h.h	Vesa */
 
-m_restart() {
-  putline("*** Oh boy... somebody wants *me* to restart... exiting...\n");
-  sleep(5);
-  exit(0);
-}
-
-#ifdef MSG_MAIL
-m_mail() {
-  putline("*** Received base message..!");
-}
-#endif
-
-m_time() {
-  putline("*** Received time message..!");
-}
-
-m_motd() {
-  putline("*** Received motd message..!");
-}
-
-m_admin() {
-  putline("*** Received admin message..!");
-}
-
-m_service() {
-  putline("*** Received service message..!");
-}
-
-m_trace() {
-  putline("*** Received trace message..!");
-}
-
-m_rehash() {
-  putline("*** Received rehash message..!");
-}
-
-m_die() {
+void m_die() {
   exit(-1);
 }
 
-m_pass() {
-  putline("*** Received Pass message !");
-}
-
-m_oper() {
-  putline("*** Received Oper message !");
-}
-
-m_names() {
-  putline("*** Received Names message !");
-}
-
-m_lusers() {
-  putline("*** Received Lusers message !");
-}
-
-m_mode(sptr, cptr, parc, parv)
+int m_mode(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -118,31 +63,30 @@ char *parv[];
     parv++;
   }
   putline(mybuf);
+  return 0;
 }
 
-m_wall(sptr, cptr, parc, parv)
+int m_wall(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
 {
   sprintf(mybuf, "*** #%s# %s", parv[0], parv[1]);
   putline(mybuf);
+  return 0;
 }
 
-m_wallops(sptr, cptr, parc, parv)
+int m_wallops(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
 {
   sprintf(mybuf, "*** =%s= %s", parv[0], parv[1]);
   putline(mybuf);
+  return 0;
 }
 
-m_connect() {
-  putline("*** Received Connect message !");
-}
-
-m_ping(sptr, cptr, parc, parv)
+int m_ping(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -152,9 +96,10 @@ char *parv[];
 	       client->sockhost, parv[2]);
   else
     sendto_one(client, "PONG %s@%s", client->user->username, client->sockhost);
+  return 0;
 }
 
-m_pong(sptr, cptr, parc, parv)
+int m_pong(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -162,6 +107,7 @@ char *parv[];
   sprintf(mybuf, "*** Received PONG message: %s %s",
 	  parv[1], (parv[2]) ? parv[2] : "");
   putline(mybuf);
+  return 0;
 }
 
 int	m_nick(sptr, cptr, parc, parv)
@@ -178,9 +124,10 @@ char	*parv[];
 #ifdef AUTOMATON
 	a_nick(parv[0], parv[1]);
 #endif
+  return 0;
 }
 
-m_away(sptr, cptr, parc, parv)
+void m_away(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -189,23 +136,7 @@ char *parv[];
   putline(mybuf);
 }
 
-m_who() { 
-  putline("*** Oh boy... server asking who from client... exiting...");
-}
-
-m_whois() {
-  putline("*** Oh boy... server asking whois from client... exiting...");
-}
-
-m_whowas() {
-  putline("*** Oh boy... server asking whowas from client... exiting...");
-}
-
-m_user() {
-  putline("*** Oh boy... server telling me user messages... exiting...");
-}
-
-m_server(sptr, cptr, parc, parv)
+void m_server(sptr, cptr, parc, parv)
 aClient *cptr, *sptr;
 int parc;
 char *parv[];
@@ -214,11 +145,7 @@ char *parv[];
   putline(mybuf);
 }
 
-m_list() {
-  putline("*** Oh boy... server asking me channel lists... exiting...");
-}
-
-m_topic(sptr, cptr, parc, parv)
+int m_topic(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -226,6 +153,7 @@ char *parv[];
   sprintf(mybuf, "*** %s changed the topic on %s to: %s",
 	  parv[0], parv[1], parv[2]);
   putline(mybuf);
+  return 0;
 }
 
 int	m_join(sptr, cptr, parc, parv)
@@ -233,12 +161,13 @@ aClient	*cptr, *sptr;
 int	parc;
 char	*parv[];
 {
-	sprintf(mybuf,"*** Change: %s has joined channel %s", 
-		parv[0], parv[1]);
+	sprintf(mybuf,"*** Change: %s <%s> has joined channel %s", 
+		parv[0], userhost, parv[1]);
 	putline(mybuf);
+  return 0;
 }
 
-m_part(sptr, cptr, parc, parv)
+int m_part(sptr, cptr, parc, parv)
 aClient *cptr, *sptr;
 int parc;
 char *parv[];
@@ -246,9 +175,10 @@ char *parv[];
   sprintf(mybuf,"*** Change: %s has left channel %s (%s)", 
 	  parv[0], parv[1], BadPtr(parv[2]) ? parv[1] : parv[2]);
   putline(mybuf);
+  return 0;
 }
 
-m_version(sptr, cptr, parc, parv)
+void m_version(sptr, cptr, parc, parv)
 aClient *cptr, *sptr;
 int parc;
 char *parv[];
@@ -257,7 +187,7 @@ char *parv[];
   putline(mybuf);
 }
 
-m_bye()
+void m_bye()
 {
 #if !defined(AUTOMATON) && !defined(VMSP)
   echo();
@@ -268,7 +198,7 @@ m_bye()
   exit(-1);    
 }
 
-m_quit(sptr, cptr, parc, parv)
+int m_quit(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -278,18 +208,20 @@ char *parv[];
 #ifdef AUTOMATON
   a_quit(sender);
 #endif
+  return 0;
 }
 
-m_kill(cptr, sptr, parc, parv)
+int m_kill(cptr, sptr, parc, parv)
 aClient *cptr, *sptr;
 int parc;
 char *parv[];
 {
   sprintf(mybuf,"*** Kill: %s (%s)", parv[0], parv[2]);
   putline(mybuf);
+  return 0;
 }
 
-m_info(sptr, cptr, parc, parv)
+void m_info(sptr, cptr, parc, parv)
 aClient *cptr, *sptr;
 int parc;
 char *parv[];
@@ -298,27 +230,7 @@ char *parv[];
   putline(mybuf);
 }
 
-m_links() { 
-  putline("*** Received LINKS message");
-}
-
-m_summon() {
-  putline("*** Received SUMMON message");
-}
-
-m_stats() {
-  putline("*** Received STATS message");
-}
-
-m_users() {
-  putline("*** Received USERS message");
-}
-
-m_help() {
-  putline("*** Received HELP message");
-}
-
-m_squit(sptr, cptr, parc, parv)
+void m_squit(sptr, cptr, parc, parv)
 aClient *cptr, *sptr;
 int parc;
 char *parv[];
@@ -327,16 +239,7 @@ char *parv[];
   putline(mybuf);
 }
 
-m_whoreply(sptr, cptr, parc, parv)
-aClient *sptr, *cptr;
-int parc;
-char *parv[];
-{
-  m_newwhoreply(parv[1], parv[2], parv[3], parv[5], parv[6], parv[7]);
-  putline(mybuf);
-}
-
-m_newwhoreply(channel, username, host, nickname, away, realname)
+void m_newwhoreply(channel, username, host, nickname, away, realname)
 char *channel, *username, *host, *nickname, *away, *realname;
 {
   /* ...dirty hack, just assume all parameters present.. --msa */
@@ -344,8 +247,7 @@ char *channel, *username, *host, *nickname, *away, *realname;
   if (*away == 'S')
     sprintf(mybuf, "  %-13s    %s %-42s %s",
 	"Nickname", "Chan", "Name", "<User@Host>");
-  else
-    {
+  else {
     int i;
     char uh[USERLEN + HOSTLEN + 1];
     if (!realname)
@@ -357,10 +259,8 @@ char *channel, *username, *host, *nickname, *away, *realname;
     if (channel[0] == '*')
 	channel = "";
 
-    if (strlen(host) > i)       /* kludge --argv */
-	{
-	host += strlen(host) - i;
-	}
+    if (strlen(host) > (size_t) i)       /* kludge --argv */
+	host += strlen(host) - (size_t) i;
 
     sprintf(uh, "%s@%s", username, host);
 
@@ -369,45 +269,10 @@ char *channel, *username, *host, *nickname, *away, *realname;
 	21-strlen(nickname)-strlen(away), channel,
 	realname,
 	53-strlen(realname), uh);
-    }
+  }
 }
 
-#if 0
-m_text(sptr, cptr, parc, parv)
-aClient *sptr, *cptr;
-int parc;
-char *parv[];
-{
-  anIgnore *iptr;
-  if ((iptr = find_ignore(parv[0], (anIgnore *)NULL, userhost)) &&
-      (iptr->flags & IGNORE_PUBLIC))
-      return(0);
-  if (!BadPtr(parv[0])) {
-    sprintf(mybuf,"<%s> %s", parv[0], parv[1]);
-#ifdef AUTOMATON
-    a_msg(mybuf);
-#else
-    putline(mybuf);
-#endif
-  } else
-    putline(parv[1]);
-}
-#endif /* 0 Vesa */
-
-m_namreply(sptr, cptr, parc, parv)
-aClient *sptr, *cptr;
-int parc;
-char *parv[];
-{
-  parv[5] = parv[4];
-  parv[4] = parv[3];
-  parv[3] = parv[2];
-  parv[2] = parv[1];
-  m_newnamreply(sptr, cptr, parc, parv);
-  putline(mybuf);
-}
-
-m_newnamreply(sptr, cptr, parc, parv)
+void m_newnamreply(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -431,7 +296,7 @@ char *parv[];
     sprintf(mybuf, "*** Internal Error: namreply");
 }
 
-m_linreply(sptr, cptr, parc, parv)
+void m_linreply(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -447,7 +312,10 @@ char	*parv[];
 	anIgnore *iptr;
 
 	iptr = find_ignore(parv[0], (anIgnore *)NULL, userhost);
-	if ((iptr != (anIgnore *)NULL) && iptr->flags & IGNORE_PRIVATE)
+	if ((iptr != (anIgnore *)NULL) &&
+	    ((iptr->flags == IGNORE_TOTAL) ||
+	     (IsChannelName(parv[1]) && (iptr->flags & IGNORE_PUBLIC)) ||
+	     (!IsChannelName(parv[1]) && (iptr->flags & IGNORE_PRIVATE))))
 		return 0;
 	check_ctcp(sptr, cptr, parc, parv);
 
@@ -467,11 +335,11 @@ char	*parv[];
 #endif
 	} else
 		putline(parv[2]); /* parv[2] and no parv[0] ?! - avalon */
-	return 0;
+  return 0;
 }
 
 
-m_kick(sptr, cptr, parc, parv)
+int m_kick(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
@@ -482,6 +350,7 @@ char *parv[];
 		(parv[1]) ? parv[1] : "*Unknown*",
 		parv[3]);
 	putline(mybuf);
+  return 0;
 }
 
 int	m_notice(sptr, cptr, parc, parv)
@@ -499,47 +368,34 @@ char	*parv[];
 		putline(mybuf);
 	} else
 		putline(parv[2]);
+  return 0;
 }
 
-m_invite(sptr, cptr, parc, parv)
+int m_invite(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
 {
   anIgnore *iptr;
   if ((iptr = find_ignore(parv[0], (anIgnore *)NULL, userhost)) &&
-      (iptr->flags & IGNORE_PRIVATE) && (iptr->flags & IGNORE_PUBLIC)) {
-	sendto_one(client,
-		   "NOTICE %s :*** Automatic reply: You have been ignored",
-		   parv[0]);
-	return(0);
-      }
+      (iptr->flags & IGNORE_PRIVATE))
+	return 0;
 #ifdef AUTOMATON
   a_invite(parv[0], parv[2]);
 #else
   sprintf(mybuf,"*** %s Invites you to channel %s", parv[0], parv[2]);
   putline(mybuf);
 #endif
-  return(0);
+  return 0;
 }
 
-m_error(sptr, cptr, parc, parv)
+int m_error(sptr, cptr, parc, parv)
 aClient *sptr, *cptr;
 int parc;
 char *parv[];
 {
   sprintf(mybuf,"*** Error: %s %s", parv[1], (parv[2]) ? parv[2] : "");
   putline(mybuf);
+  return 0;
 }
 
-m_userhost() {
-  putline("*** Oh boy... server asking userhost from client... exiting...");
-}
-
-m_ison() {
-  putline("*** Oh boy... server asking ison from client... exiting...");
-}
-
-m_close() {
-  putline("*** Oh boy... server asking close from client... exiting...");
-}
