@@ -35,7 +35,7 @@
  */
 
 #ifndef lint
-static  char sccsid[] = "@(#)s_bsd.c	2.68 07 Nov 1993 (C) 1988 University of Oulu, \
+static  char sccsid[] = "@(#)s_bsd.c	2.78 2/7/94 (C) 1988 University of Oulu, \
 Computing Center and Jarkko Oikarinen";
 #endif
 
@@ -48,6 +48,9 @@ Computing Center and Jarkko Oikarinen";
 #include <sys/socket.h>
 #include <sys/file.h>
 #include <sys/ioctl.h>
+#if defined(SOL20)
+#include <sys/filio.h>
+#endif
 #if defined(UNIXPORT) && (!defined(SVR3) || defined(sgi) || \
     defined(_SEQUENT_))
 # include <sys/un.h>
@@ -405,9 +408,8 @@ void	close_listeners()
 		if (!IsMe(cptr) || cptr == &me || !IsListening(cptr))
 			continue;
 		aconf = cptr->confs->value.aconf;
-		aconf->status |= CONF_ILLEGAL;
 
-		if (aconf->clients == 0)
+		if (IsIllegal(aconf) && aconf->clients == 0)
 		    {
 #ifdef	UNIXPORT
 			if (IsUnixSocket(cptr))
@@ -471,7 +473,7 @@ void	init_sys()
 	    }
 # endif
 #endif
-#if defined(PCS) || defined(DYNIXPTX)
+#if defined(PCS) || defined(DYNIXPTX) || defined(SVR3)
 	char	logbuf[BUFSIZ];
 
 	(void)setvbuf(stderr,logbuf,_IOLBF,sizeof(logbuf));
@@ -635,11 +637,11 @@ Reg1	aClient	*cptr;
 		    }
 	    }
 
-	if (attach_Iline(cptr, hp, sockname))
+	if ((i = attach_Iline(cptr, hp, sockname)))
 	    {
 		Debug((DEBUG_DNS,"ch_cl: access denied: %s[%s]",
 			cptr->name, sockname));
-		return -1;
+		return i;
 	    }
 
 	Debug((DEBUG_DNS, "ch_cl: access ok: %s[%s]",
@@ -940,14 +942,38 @@ aClient *cptr;
 		ircstp->is_sv++;
 		ircstp->is_sbs += cptr->sendB;
 		ircstp->is_sbr += cptr->receiveB;
+		ircstp->is_sks += cptr->sendK;
+		ircstp->is_skr += cptr->receiveK;
 		ircstp->is_sti += time(NULL) - cptr->firsttime;
+		if (ircstp->is_sbs > 1023)
+		    {
+			ircstp->is_sks += (ircstp->is_sbs >> 10);
+			ircstp->is_sbs &= 0x3ff;
+		    }
+		if (ircstp->is_sbr > 1023)
+		    {
+			ircstp->is_skr += (ircstp->is_sbr >> 10);
+			ircstp->is_sbr &= 0x3ff;
+		    }
 	    }
 	else if (IsClient(cptr))
 	    {
 		ircstp->is_cl++;
 		ircstp->is_cbs += cptr->sendB;
 		ircstp->is_cbr += cptr->receiveB;
+		ircstp->is_cks += cptr->sendK;
+		ircstp->is_ckr += cptr->receiveK;
 		ircstp->is_cti += time(NULL) - cptr->firsttime;
+		if (ircstp->is_cbs > 1023)
+		    {
+			ircstp->is_cks += (ircstp->is_cbs >> 10);
+			ircstp->is_cbs &= 0x3ff;
+		    }
+		if (ircstp->is_cbr > 1023)
+		    {
+			ircstp->is_ckr += (ircstp->is_cbr >> 10);
+			ircstp->is_cbr &= 0x3ff;
+		    }
 	    }
 	else
 		ircstp->is_ni++;
@@ -992,7 +1018,7 @@ aClient *cptr;
 		 * clean up extra sockets from P-lines which have been
 		 * discarded.
 		 */
-		if (cptr->acpt != &me)
+		if (cptr->acpt != &me && cptr->acpt != cptr)
 		    {
 			aconf = cptr->acpt->confs->value.aconf;
 			if (aconf->clients > 0)
@@ -1040,10 +1066,13 @@ aClient	*cptr;
 	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)) < 0)
 		report_error("setsockopt(SO_REUSEADDR) %s:%s", cptr);
 #endif
-#if  defined(SO_DEBUG) && defined(DEBUGMODE)
+#if  defined(SO_DEBUG) && defined(DEBUGMODE) && 0
+/* Solaris with SO_DEBUG writes to syslog by default */
+#if !defined(SOL20) || defined(USE_SYSLOG)
 	opt = 1;
 	if (setsockopt(fd, SOL_SOCKET, SO_DEBUG, &opt, sizeof(opt)) < 0)
 		report_error("setsockopt(SO_DEBUG) %s:%s", cptr);
+#endif /* SOL20 */
 #endif
 #ifdef	SO_USELOOPBACK
 	opt = 1;
@@ -1067,8 +1096,28 @@ aClient	*cptr;
 	if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &opt, sizeof(opt)) < 0)
 		report_error("setsockopt(SO_SNDBUF) %s:%s", cptr);
 #endif
+#if defined(IP_OPTIONS) && defined(IPPROTO_IP)
+	{
+	char	*s = readbuf, *t = readbuf + sizeof(readbuf) / 2;
+
+	opt = sizeof(readbuf) / 8;
+	if (getsockopt(fd, IPPROTO_IP, IP_OPTIONS, t, &opt) < 0)
+		report_error("getsockopt(IP_OPTIONS) %s:%s", cptr);
+	else if (opt > 0)
+	    {
+		for (*readbuf = '\0'; opt > 0; opt--, s+= 3)
+			(void)sprintf(s, "%02.2x:", *t++);
+		*s = '\0';
+		sendto_ops("Connection %s using IP opts: (%s)",
+			   get_client_name(cptr, TRUE), readbuf);
+	    }
+	if (setsockopt(fd, IPPROTO_IP, IP_OPTIONS, (char *)NULL, 0) < 0)
+		report_error("setsockopt(IP_OPTIONS) %s:%s", cptr);
+	}
+#endif
 }
 
+
 int	get_sockerr(cptr)
 aClient	*cptr;
 {
@@ -1286,7 +1335,8 @@ fd_set	*rfd;
 		/*
 		 * If not ready, fake it so it isnt closed
 		 */
-		if (length == -1 && errno == EWOULDBLOCK)
+		if (length == -1 &&
+			((errno == EWOULDBLOCK) || (errno == EAGAIN)))
 			return 1;
 		if (length <= 0)
 			return length;
@@ -1382,15 +1432,26 @@ time_t	delay; /* Don't ever use ZERO here, unless you mean to poll and then
 	Reg1	aClient	*cptr;
 	Reg2	int	nfds;
 	struct	timeval	wait;
+#ifdef	pyr
+	struct	timeval	nowt;
+	u_long	us;
+#endif
 	fd_set	read_set, write_set;
 	time_t	delay2 = delay, now;
+	u_long	usec = 0;
 	int	res, length, fd, i;
 	int	auth = 0;
 
 #ifdef NPATH
 	check_command(&delay, NULL);
 #endif
+#ifdef	pyr
+	(void) gettimeofday(&nowt, NULL);
+	now = nowt.tv_sec;
+#else
 	now = time(NULL);
+#endif
+
 	for (res = 0;;)
 	    {
 		FD_ZERO(&read_set);
@@ -1428,7 +1489,24 @@ time_t	delay; /* Don't ever use ZERO here, unless you mean to poll and then
 			    }
 
 			if (DBufLength(&cptr->sendQ) || IsConnecting(cptr))
+#ifndef	pyr
 				FD_SET(i, &write_set);
+#else
+			    {
+				if (!(cptr->flags & FLAGS_BLOCKED))
+					FD_SET(i, &write_set);
+				else
+					delay2 = 0, usec = 500000;
+			    }
+			if (now - cptr->lw.tv_sec &&
+			    nowt.tv_usec - cptr->lw.tv_usec < 0)
+				us = 1000000;
+			else
+				us = 0;
+			us += nowt.tv_usec;
+			if (us - cptr->lw.tv_usec > 500000)
+				cptr->flags &= ~FLAGS_BLOCKED;
+#endif
 		    }
 
 		if (udpfd >= 0)
@@ -1437,7 +1515,7 @@ time_t	delay; /* Don't ever use ZERO here, unless you mean to poll and then
 			FD_SET(resfd, &read_set);
 
 		wait.tv_sec = MIN(delay2, delay);
-		wait.tv_usec = 0;
+		wait.tv_usec = usec;
 #ifdef	HPUX
 		nfds = select(FD_SETSIZE, (int *)&read_set, (int *)&write_set,
 				0, &wait);
@@ -1521,6 +1599,9 @@ time_t	delay; /* Don't ever use ZERO here, unless you mean to poll and then
 				ircstp->is_ref++;
 				sendto_ops("All connections in use. (%s)",
 					   get_client_name(cptr, TRUE));
+				(void)send(fd,
+					"ERROR :All connections in use\r\n",
+					32, 0);
 				(void)close(fd);
 				break;
 			    }
@@ -1631,6 +1712,11 @@ struct	hostent	*hp;
 	    {
 		sendto_ops("Server %s already present from %s",
 			   aconf->name, get_client_name(c2ptr, TRUE));
+		if (by && IsPerson(by) && !MyClient(by))
+		  sendto_one(by,
+                             ":%s NOTICE %s :Server %s already present from %s",
+                             me.name, by->name, aconf->name,
+			     get_client_name(c2ptr, TRUE));
 		return -1;
 	    }
 
@@ -1638,7 +1724,11 @@ struct	hostent	*hp;
 	 * If we dont know the IP# for this host and itis a hostname and
 	 * not a ip# string, then try and find the appropriate host record.
 	 */
-	if (!aconf->ipnum.s_addr)
+	if ( ( !aconf->ipnum.s_addr )
+#ifdef UNIXPORT
+	    && ( ( aconf->host[2] ) != '/' )  /* needed for Unix domain -- dl*/
+#endif
+            )
 	    {
 	        Link    lin;
 
@@ -1668,7 +1758,7 @@ struct	hostent	*hp;
 	strncpyzt(cptr->sockhost, aconf->host, HOSTLEN+1);
 
 #ifdef	UNIXPORT
-	if (aconf->host[0] == '/')
+	if (aconf->host[2] == '/') /* (/ starts a 2), Unix domain -- dl*/
 		svp = connect_unix(aconf, cptr, &len);
 	else
 		svp = connect_inet(aconf, cptr, &len);
@@ -1694,6 +1784,10 @@ struct	hostent	*hp;
 		errtmp = errno; /* other system calls may eat errno */
 		(void)alarm(0);
 		report_error("Connect to host %s failed: %s",cptr);
+                if (by && IsPerson(by) && !MyClient(by))
+                  sendto_one(by,
+                             ":%s NOTICE %s :Connect to host %s failed.",
+			     me.name, by->name, cptr);
 		(void)close(cptr->fd);
 		cptr->fd = -2;
 		free_client(cptr);
@@ -1719,6 +1813,10 @@ struct	hostent	*hp;
 	    {
       		sendto_ops("Host %s is not enabled for connecting:no C/N-line",
 			   aconf->host);
+                if (by && IsPerson(by) && !MyClient(by))
+                  sendto_one(by,
+                             ":%s NOTICE %s :Connect to host %s failed.",
+			     me.name, by->name, cptr);
 		det_confs_butmask(cptr, 0);
 		(void)close(cptr->fd);
 		cptr->fd = -2;
@@ -1734,7 +1832,8 @@ struct	hostent	*hp;
 		(void)strcpy(cptr->serv->by, by->name);
 		cptr->serv->user = by->user;
 		by->user->refcnt++;
-	    }
+	    } else
+		(void)strcpy(cptr->serv->by, "AutoConn.");
 	(void)strcpy(cptr->serv->up, me.name);
 	if (cptr->fd > highest_fd)
 		highest_fd = cptr->fd;
@@ -1829,7 +1928,7 @@ int	*lenp;
 
 	if ((cptr->fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
 	    {
-		report_error("Connect to host %s failed: %s", cptr);
+		report_error("Unix domain connect to host %s failed: %s", cptr);
 		return NULL;
 	    }
 	else if (cptr->fd >= MAXCLIENTS)
@@ -1839,7 +1938,8 @@ int	*lenp;
 	    }
 
 	get_sockhost(cptr, aconf->host);
-	strncpyzt(sock.sun_path, aconf->host, sizeof(sock.sun_path));
+	/* +2 needed for working Unix domain -- dl*/
+	strncpyzt(sock.sun_path, aconf->host+2, sizeof(sock.sun_path));
 	sock.sun_family = AF_UNIX;
 	*lenp = strlen(sock.sun_path) + 2;
 
@@ -2095,21 +2195,31 @@ int	setup_ping()
 	if (setsockopt(udpfd, SOL_SOCKET, SO_REUSEADDR,
 			(char *)&on, sizeof(on)) == -1)
 	    {
+#ifdef	USE_SYSLOG
+		syslog(LOG_ERR, "setsockopt udp fd %d : %m", udpfd);
+#endif
 		Debug((DEBUG_ERROR, "setsockopt so_reuseaddr : %s",
 			strerror(errno)));
 		(void)close(udpfd);
+		udpfd = -1;
 		return -1;
 	    }
 	if (bind(udpfd, (struct sockaddr *)&from, sizeof(from))==-1)
 	    {
+#ifdef	USE_SYSLOG
+		syslog(LOG_ERR, "bind udp.%d fd %d : %m",
+			from.sin_port, udpfd);
+#endif
 		Debug((DEBUG_ERROR, "bind : %s", strerror(errno)));
 		(void)close(udpfd);
+		udpfd = -1;
 		return -1;
 	    }
 	if (fcntl(udpfd, F_SETFL, FNDELAY)==-1)
 	    {
 		Debug((DEBUG_ERROR, "fcntl fndelay : %s", strerror(errno)));
 		(void)close(udpfd);
+		udpfd = -1;
 		return -1;
 	    }
 	return udpfd;
@@ -2137,6 +2247,8 @@ static	void	polludp()
 			mlen = 0;
 	    }
 	Debug((DEBUG_DEBUG,"udp poll"));
+
+	n = recvfrom(udpfd, readbuf, mlen, 0, &from, &fromlen);
 	now = time(NULL);
 	if (now == last)
 		if (++cnt > 14)
@@ -2144,14 +2256,13 @@ static	void	polludp()
 	cnt = 0;
 	last = now;
 
-	n = recvfrom(udpfd, readbuf, sizeof(readbuf), 0, &from, &fromlen);
 	if (n == -1)
 	    {
-		if (errno == EWOULDBLOCK)
+		if ((errno == EWOULDBLOCK) || (errno == EAGAIN))
 			return;
 		else
 		    {
-			report_error("udp port recvfrom : %s", &me);
+			report_error("udp port recvfrom (%s): %s", &me);
 			return;
 		    }
 	    }
@@ -2159,13 +2270,11 @@ static	void	polludp()
 	if (n  < 8)
 		return;
 
-	/*
-	 * first 8 bytes...sequence # and 4 bytes data..
-	 */
-	s = readbuf + 12;
+	s = readbuf + n;
 	/*
 	 * attach my name and version for the reply
 	 */
+	*readbuf |= 1;
 	(void)strcpy(s, me.name);
 	s += strlen(s)+1;
 	(void)strcpy(s, PATCHLEVEL);
@@ -2203,6 +2312,7 @@ static	void	do_dns_async()
 	case ASYNC_CLIENT :
 		if ((cptr = ln.value.cptr))
 		    {
+			del_queries((char *)cptr);
 			ClearDNS(cptr);
 			if (!DoingAuth(cptr))
 				SetAccess(cptr);
@@ -2229,6 +2339,7 @@ static	void	do_dns_async()
 		break;
 	case ASYNC_SERVER :
 		cptr = ln.value.cptr;
+		del_queries((char *)cptr);
 		ClearDNS(cptr);
 		if (check_server(cptr, hp, NULL, NULL, 1))
 			(void)exit_client(cptr, cptr, &me,
