@@ -48,7 +48,7 @@
  */
 
 #ifndef lint
-static  char sccsid[] = "@(#)s_conf.c	2.49 27 Oct 1993 (C) 1988 University of Oulu, \
+static  char sccsid[] = "@(#)s_conf.c	2.56 02 Apr 1994 (C) 1988 University of Oulu, \
 Computing Center and Jarkko Oikarinen";
 #endif
 
@@ -62,7 +62,7 @@ Computing Center and Jarkko Oikarinen";
 #ifdef __hpux
 #include "inet.h"
 #endif
-#if defined(PCS) || defined(AIX) || defined(DYNIXPTX)
+#if defined(PCS) || defined(AIX) || defined(DYNIXPTX) || defined(SVR3)
 #include <time.h>
 #endif
 #ifdef	R_LINES
@@ -135,7 +135,7 @@ char	*sockhost;
 					*uhost = '\0';
 				(void)strncat(uhost, fullname,
 					sizeof(uhost) - strlen(uhost));
-				if (!matches(aconf->name, uhost))
+				if (!match(aconf->name, uhost))
 					goto attach_iline;
 			    }
 
@@ -147,7 +147,7 @@ char	*sockhost;
 		else
 			*uhost = '\0';
 		(void)strncat(uhost, sockhost, sizeof(uhost) - strlen(uhost));
-		if (!matches(aconf->host, uhost))
+		if (!match(aconf->host, uhost))
 			goto attach_iline;
 		continue;
 attach_iline:
@@ -255,7 +255,7 @@ aClient *cptr;
 		return -1;
 	if ((aconf->status & (CONF_LOCOP | CONF_OPERATOR | CONF_CLIENT)) &&
 	    aconf->clients >= ConfMaxLinks(aconf) && ConfMaxLinks(aconf) > 0)
-		return -1;
+		return -3;	/* Use this for printing error message */
 	lp = make_link();
 	lp->next = cptr->confs;
 	lp->value.aconf = aconf;
@@ -309,7 +309,7 @@ int	statmask;
 	    {
 		if ((tmp->status & statmask) && !IsIllegal(tmp) &&
 		    ((tmp->status & (CONF_SERVER_MASK|CONF_HUB)) == 0) &&
-		    tmp->name && !matches(tmp->name, name))
+		    tmp->name && !match(tmp->name, name))
 		    {
 			if (!attach_conf(cptr, tmp) && !first)
 				first = tmp;
@@ -344,7 +344,7 @@ int	statmask;
 	    {
 		if ((tmp->status & statmask) && !IsIllegal(tmp) &&
 		    (tmp->status & CONF_SERVER_MASK) == 0 &&
-		    (!tmp->host || matches(tmp->host, host) == 0))
+		    (!tmp->host || match(tmp->host, host) == 0))
 		    {
 			if (!attach_conf(cptr, tmp) && !first)
 				first = tmp;
@@ -382,7 +382,7 @@ int	statmask;
 		** socket host) matches *either* host or name field
 		** of the configuration.
 		*/
-		if (matches(tmp->host, userhost))
+		if (match(tmp->host, userhost))
 			continue;
 		if (tmp->status & (CONF_OPERATOR|CONF_LOCOP))
 		    {
@@ -410,7 +410,7 @@ int	statmask;
 		** matches *either* host or name field of the configuration.
 		*/
 		if ((tmp->status & statmask) &&
-		    (!tmp->name || matches(tmp->name, name) == 0))
+		    (!tmp->name || match(tmp->name, name) == 0))
 			return tmp;
 	    }
 	return NULL;
@@ -434,7 +434,7 @@ int	statmask;
 		    (((tmp->status & (CONF_SERVER_MASK|CONF_HUB)) &&
 	 	     tmp->name && !mycmp(tmp->name, name)) ||
 		     ((tmp->status & (CONF_SERVER_MASK|CONF_HUB)) == 0 &&
-		     tmp->name && !matches(tmp->name, name))))
+		     tmp->name && !match(tmp->name, name))))
 			return tmp;
 	    }
 	return NULL;
@@ -458,7 +458,7 @@ Reg3	int	statmask;
 		tmp = lp->value.aconf;
 		if (tmp->status & statmask &&
 		    (!(tmp->status & CONF_SERVER_MASK || tmp->host) ||
-	 	     (tmp->host && !matches(tmp->host, host))))
+	 	     (tmp->host && !match(tmp->host, host))))
 			return tmp;
 	    }
 	return NULL;
@@ -485,7 +485,7 @@ int	statmask;
 			continue;
 		s = index(tmp->host, '@');
 		*s = '\0';
-		if (matches(tmp->host, user))
+		if (match(tmp->host, user))
 		    {
 			*s = '@';
 			continue;
@@ -504,10 +504,11 @@ int	statmask;
  */
 aConfItem *find_conf_entry(aconf, mask)
 aConfItem *aconf;
+u_int	mask;
 {
 	Reg1	aConfItem *bconf;
 
-	for (bconf = conf; bconf; bconf = bconf->next)
+	for (bconf = conf, mask &= ~CONF_ILLEGAL; bconf; bconf = bconf->next)
 	    {
 		if (!(bconf->status & mask) || (bconf->port != aconf->port))
 			continue;
@@ -552,7 +553,7 @@ int	sig;
 	Reg2	int	i;
 	int	ret = 0;
 
-	if (sig)
+	if (sig == 1)
 	    {
 		sendto_ops("Got signal SIGHUP, reloading ircd conf. file");
 #ifdef	ULTRIX
@@ -585,8 +586,7 @@ int	sig;
 		    }
 
 	while ((tmp2 = *tmp))
-	    {
-		if (tmp2->clients)
+		if (tmp2->clients || tmp2->status & CONF_LISTEN_PORT)
 		    {
 			/*
 			** Configuration entry is still in use by some
@@ -594,7 +594,7 @@ int	sig;
 			** that it will be deleted when the last client
 			** exits...
 			*/
-			if (!(tmp2->status & CONF_LISTEN_PORT))
+			if (!(tmp2->status & (CONF_LISTEN_PORT|CONF_CLIENT)))
 			    {
 				*tmp = tmp2->next;
 				tmp2->next = NULL;
@@ -608,7 +608,6 @@ int	sig;
 			*tmp = tmp2->next;
 			free_conf(tmp2);
 	    	    }
-	    }
 
 	/*
 	 * We don't delete the class table, rather mark all entries
@@ -617,9 +616,24 @@ int	sig;
 	for (cltmp = NextClass(FirstClass()); cltmp; cltmp = NextClass(cltmp))
 		MaxLinks(cltmp) = -1;
 
-	close_listeners();
-	flush_cache();
+	if (sig != 2)
+		flush_cache();
 	(void) initconf(0);
+	close_listeners();
+
+	/*
+	 * flush out deleted I and P lines although still in use.
+	 */
+	for (tmp = &conf; (tmp2 = *tmp); )
+		if (!(tmp2->status & CONF_ILLEGAL))
+			tmp = &tmp2->next;
+		else
+		    {
+			*tmp = tmp2->next;
+			tmp2->next = NULL;
+			if (!tmp2->clients)
+				free_conf(tmp2);
+		    }
 	return ret;
 }
 
@@ -742,10 +756,7 @@ int	opt;
                         continue;
                     }
 		if (aconf)
-		    {
 			free_conf(aconf);
-			aconf = NULL;
-		    }
 		aconf = make_conf();
 
 		tmp = getfield(line);
@@ -863,16 +874,36 @@ int	opt;
 				  tmp ? atoi(tmp) : 0);
 			continue;
 		    }
-		if (aconf->status & CONF_LISTEN_PORT)
+		/*
+                ** associate each conf line with a class by using a pointer
+                ** to the correct class record. -avalon
+                */
+		if (aconf->status & (CONF_CLIENT_MASK|CONF_LISTEN_PORT))
+		    {
+			if (Class(aconf) == 0)
+				Class(aconf) = find_class(0);
+			if (MaxLinks(Class(aconf)) < 0)
+				Class(aconf) = find_class(0);
+		    }
+		if (aconf->status & (CONF_LISTEN_PORT|CONF_CLIENT))
 		    {
 			aConfItem *bconf;
 
-			if (bconf = find_conf_entry(aconf, CONF_LISTEN_PORT))
+			if (bconf = find_conf_entry(aconf, aconf->status))
 			    {
+				delist_conf(bconf);
 				bconf->status &= ~CONF_ILLEGAL;
-				continue;
+				if (aconf->status == CONF_CLIENT)
+				    {
+					bconf->class->links -= bconf->clients;
+					bconf->class = aconf->class;
+					bconf->class->links += bconf->clients;
+				    }
+				free_conf(aconf);
+				aconf = bconf;
 			    }
-			else if (aconf->host)
+			else if (aconf->host &&
+				 aconf->status == CONF_LISTEN_PORT)
 				(void)add_listener(aconf);
 		    }
 		if (aconf->status & CONF_SERVER_MASK)
@@ -894,17 +925,6 @@ int	opt;
 				MyFree(aconf->host);
 				aconf->host = newhost;
 			    }
-		/*
-                ** associate each conf line with a class by using a pointer
-                ** to the correct class record. -avalon
-                */
-		if (aconf->status & CONF_CLIENT_MASK)
-		    {
-			if (Class(aconf) == 0)
-				Class(aconf) = find_class(0);
-			if (MaxLinks(Class(aconf)) < 0)
-				Class(aconf) = find_class(0);
-		    }
 		if (aconf->status & CONF_SERVER_MASK)
 		    {
 			if (BadPtr(aconf->passwd))
@@ -929,6 +949,8 @@ int	opt;
 			if (portnum < 0 && aconf->port >= 0)
 				portnum = aconf->port;
 		    }
+		(void)collapse(aconf->host);
+		(void)collapse(aconf->name);
 		Debug((DEBUG_NOTICE,
 		      "Read Init: (%d) (%s) (%s) (%s) (%d) (%d)",
 		      aconf->status, aconf->host, aconf->passwd,
@@ -1018,8 +1040,9 @@ aClient	*cptr;
 
 	for (tmp = conf; tmp; tmp = tmp->next)
  		if ((tmp->status == CONF_KILL) && tmp->host && tmp->name &&
-		    (matches(tmp->host, host) == 0) &&
- 		    (!name || matches(tmp->name, name) == 0))
+		    (match(tmp->host, host) == 0) &&
+ 		    (!name || match(tmp->name, name) == 0) &&
+		    (!tmp->port || (tmp->port == cptr->acpt->port)))
  			if (BadPtr(tmp->passwd) ||
  			    check_time_interval(tmp->passwd, reply))
  			break;
@@ -1028,9 +1051,8 @@ aClient	*cptr;
 		sendto_one(cptr, reply,
 			   me.name, ERR_YOUREBANNEDCREEP, cptr->name);
 	else if (tmp)
-		sendto_one(cptr,
-			   ":%s %d %s :*** Ghosts are not allowed on IRC.",
-			   me.name, ERR_YOUREBANNEDCREEP, cptr->name);
+		sendto_one(cptr, err_str(ERR_YOUREBANNEDCREEP), me.name,
+			   cptr->name);
 
  	return (tmp ? -1 : 0);
  }
@@ -1063,8 +1085,8 @@ aClient	*cptr;
 	for (tmp = conf; tmp; tmp = tmp->next)
 	    {
 		if (tmp->status != CONF_RESTRICT ||
-		    (tmp->host && host && matches(tmp->host, host)) ||
-		    (tmp->name && name && matches(tmp->name, name)))
+		    (tmp->host && host && match(tmp->host, host)) ||
+		    (tmp->name && name && match(tmp->name, name)))
 			continue;
 
 		if (BadPtr(tmp->passwd))
